# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._types import SequenceNotStr, Base64FileInput
from .._utils import PropertyInfo
from .._models import set_pydantic_config
from .content_block import ContentBlock
from .request_text_block_param import RequestTextBlockParam
from .request_image_block_param import RequestImageBlockParam
from .cache_control_ephemeral_param import CacheControlEphemeralParam

__all__ = [
    "MessageCreateParams",
    "Message",
    "MessageContentUnionMember1AnthropicRequestDocumentBlock",
    "MessageContentUnionMember1AnthropicRequestDocumentBlockSource",
    "MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicBase64PdfSource",
    "MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicPlainTextSource",
    "MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicContentBlockSource",
    "MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicContentBlockSourceContentContentBlockSourceContent",
    "MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicUrlpdfSource",
    "MessageContentUnionMember1AnthropicRequestDocumentBlockCitations",
    "MessageContentUnionMember1AnthropicRequestThinkingBlock",
    "MessageContentUnionMember1AnthropicRequestRedactedThinkingBlock",
    "MessageContentUnionMember1AnthropicRequestToolUseBlock",
    "MessageContentUnionMember1AnthropicRequestToolResultBlock",
    "Content",
    "ContentAnthropicRequestDocumentBlock",
    "ContentAnthropicRequestDocumentBlockSource",
    "ContentAnthropicRequestDocumentBlockSourceAnthropicBase64PdfSource",
    "ContentAnthropicRequestDocumentBlockSourceAnthropicPlainTextSource",
    "ContentAnthropicRequestDocumentBlockSourceAnthropicContentBlockSource",
    "ContentAnthropicRequestDocumentBlockSourceAnthropicContentBlockSourceContentContentBlockSourceContent",
    "ContentAnthropicRequestDocumentBlockSourceAnthropicUrlpdfSource",
    "ContentAnthropicRequestDocumentBlockCitations",
    "Metadata",
    "OutputConfig",
    "OutputConfigFormat",
    "Thinking",
    "ThinkingAnthropicThinkingConfigEnabled",
    "ThinkingAnthropicThinkingConfigDisabled",
    "ThinkingAnthropicThinkingConfigAdaptive",
    "ToolChoice",
    "ToolChoiceAnthropicToolChoiceAuto",
    "ToolChoiceAnthropicToolChoiceAny",
    "ToolChoiceAnthropicToolChoiceTool",
    "ToolChoiceAnthropicToolChoiceNone",
    "Tool",
    "ToolInputSchema",
]


class MessageCreateParams(TypedDict, total=False):
    messages: Required[Iterable[Message]]
    """Input messages.

    Models are trained to operate on alternating `user` and `assistant`
    conversational turns. When creating a new `Message`, you specify the prior
    conversational turns with the `messages` parameter, and the model then generates
    the next `Message` in the conversation. Consecutive `user` or `assistant` turns
    in your request will be combined into a single turn.

    Each input message must be an object with a `role` and `content`. You can
    specify a single `user`-role message, or you can include multiple `user` and
    `assistant` messages.

    If the final message uses the `assistant` role, the response content will
    continue immediately from the content in that message. This can be used to
    constrain part of the model's response.

    Example with a single `user` message:

    ```json
    [{ "role": "user", "content": "Hello" }]
    ```

    Example with multiple conversational turns:

    ```json
    [
      { "role": "user", "content": "Hello there." },
      {
        "role": "assistant",
        "content": "Hi, I'm here to help. How can I help you?"
      },
      { "role": "user", "content": "Can you explain LLMs in plain English?" }
    ]
    ```

    Example with a partially-filled response from the model:

    ```json
    [
      {
        "role": "user",
        "content": "What's the Greek name for Sun? (A) Sol (B) Helios (C) Sun"
      },
      { "role": "assistant", "content": "The best answer is (" }
    ]
    ```

    Each input message `content` may be either a single `string` or an array of
    content blocks, where each block has a specific `type`. Using a `string` for
    `content` is shorthand for an array of one content block of type `"text"`. The
    following input messages are equivalent:

    ```json
    { "role": "user", "content": "Hello" }
    ```

    ```json
    { "role": "user", "content": [{ "type": "text", "text": "Hello" }] }
    ```

    See [input examples](https://docs.claude.com/en/api/messages-examples).

    Note that if you want to include a
    [system prompt](/guides/querying-text-models), you can use the top-level
    `system` parameter — there is no `"system"` role for input messages in the
    Messages API.

    There is a limit of 100,000 messages in a single request.
    """

    model: Required[str]
    """The model that will complete your prompt.

    See the [Fireworks Model Library](https://app.fireworks.ai/models) for available
    models.
    """

    max_tokens: int
    """The maximum number of tokens to generate before stopping.

    Note that models may stop _before_ reaching this maximum. This parameter only
    specifies the absolute maximum number of tokens to generate.

    Different models have different maximum values for this parameter. See
    [models](https://app.fireworks.ai/models) for details.
    """

    metadata: Metadata
    """An object describing metadata about the request."""

    output_config: OutputConfig
    """Configuration options for the model's output, such as the output format."""

    raw_output: Optional[bool]
    """Return raw output from the model."""

    stop_sequences: SequenceNotStr[str]
    """Custom text sequences that will cause the model to stop generating.

    Models will normally stop when they have naturally completed their turn, which
    will result in a response `stop_reason` of `"end_turn"`.

    If you want the model to stop generating when it encounters custom strings of
    text, you can use the `stop_sequences` parameter. If the model encounters one of
    the custom sequences, the response `stop_reason` value will be `"stop_sequence"`
    and the response `stop_sequence` value will contain the matched stop sequence.
    """

    stream: bool
    """Whether to incrementally stream the response using server-sent events.

    See [streaming](/guides/querying-text-models) for details.
    """

    system: Union[str, Iterable[RequestTextBlockParam]]
    """System prompt.

    A system prompt is a way of providing context and instructions to the model,
    such as specifying a particular goal or role. See the
    [guide to system prompts](/guides/querying-text-models).
    """

    temperature: float
    """Amount of randomness injected into the response.

    Defaults to `1.0`. Ranges from `0.0` to `1.0`. Use `temperature` closer to `0.0`
    for analytical / multiple choice, and closer to `1.0` for creative and
    generative tasks.

    Note that even with `temperature` of `0.0`, the results will not be fully
    deterministic.
    """

    thinking: Thinking
    """Configuration for enabling the model's extended thinking.

    When enabled, responses include `thinking` content blocks showing the model's
    thinking process before the final answer. Requires a minimum budget of 1,024
    tokens and counts towards your `max_tokens` limit.

    See [reasoning](/guides/reasoning) for details.

    **Note:** The `adaptive` thinking type is not supported yet.
    """

    tool_choice: ToolChoice
    """How the model should use the provided tools.

    The model can use a specific tool, any available tool, decide by itself, or not
    use tools at all.
    """

    tools: Iterable[Tool]
    """Definitions of tools that the model may use.

    If you include `tools` in your API request, the model may return `tool_use`
    content blocks that represent the model's use of those tools. You can then run
    those tools using the tool input generated by the model and then optionally
    return results back to the model using `tool_result` content blocks.

    Each tool definition includes:

    - `name`: Name of the tool.
    - `description`: Optional, but strongly-recommended description of the tool.
    - `input_schema`: [JSON schema](https://json-schema.org/draft/2020-12) for the
      tool `input` shape that the model will produce in `tool_use` output content
      blocks.

    For example, if you defined `tools` as:

    ```json
    [
      {
        "name": "get_stock_price",
        "description": "Get the current stock price for a given ticker symbol.",
        "input_schema": {
          "type": "object",
          "properties": {
            "ticker": {
              "type": "string",
              "description": "The stock ticker symbol, e.g. AAPL for Apple Inc."
            }
          },
          "required": ["ticker"]
        }
      }
    ]
    ```

    And then asked the model "What's the S&P 500 at today?", the model might produce
    `tool_use` content blocks in the response like this:

    ```json
    [
      {
        "type": "tool_use",
        "id": "toolu_01D7FLrfh4GYq7yT1ULFeyMV",
        "name": "get_stock_price",
        "input": { "ticker": "^GSPC" }
      }
    ]
    ```

    You might then run your `get_stock_price` tool with `{"ticker": "^GSPC"}` as an
    input, and return the following back to the model in a subsequent `user`
    message:

    ```json
    [
      {
        "type": "tool_result",
        "tool_use_id": "toolu_01D7FLrfh4GYq7yT1ULFeyMV",
        "content": "259.75 USD"
      }
    ]
    ```

    Tools can be used for workflows that include running client-side tools and
    functions, or more generally whenever you want the model to produce a particular
    JSON structure of output.

    See the [guide](/guides/function-calling) for more details.
    """

    top_k: int
    """Only sample from the top K options for each subsequent token.

    Used to remove "long tail" low probability responses.
    [Learn more technical details here](https://towardsdatascience.com/how-to-sample-from-language-models-682bceb97277).

    Recommended for advanced use cases only. You usually only need to use
    `temperature`.
    """

    top_p: float
    """Use nucleus sampling.

    In nucleus sampling, we compute the cumulative distribution over all the options
    for each subsequent token in decreasing probability order and cut it off once it
    reaches a particular probability specified by `top_p`. You should either alter
    `temperature` or `top_p`, but not both.

    Recommended for advanced use cases only. You usually only need to use
    `temperature`.
    """


class MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicBase64PdfSource(
    TypedDict, total=False
):
    data: Required[
        Annotated[Union[str, Base64FileInput], PropertyInfo(format="base64")]
    ]

    media_type: Required[Literal["application/pdf"]]

    type: Required[Literal["base64"]]


set_pydantic_config(
    MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicBase64PdfSource,
    {"arbitrary_types_allowed": True},
)


class MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicPlainTextSource(
    TypedDict, total=False
):
    data: Required[str]

    media_type: Required[Literal["text/plain"]]

    type: Required[Literal["text"]]


MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicContentBlockSourceContentContentBlockSourceContent: (
    TypeAlias
) = Union[
    RequestTextBlockParam, RequestImageBlockParam
]


class MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicContentBlockSource(
    TypedDict, total=False
):
    content: Required[
        Union[
            str,
            Iterable[
                MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicContentBlockSourceContentContentBlockSourceContent
            ],
        ]
    ]

    type: Required[Literal["content"]]


class MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicUrlpdfSource(
    TypedDict, total=False
):
    type: Required[Literal["url"]]

    url: Required[str]


MessageContentUnionMember1AnthropicRequestDocumentBlockSource: TypeAlias = Union[
    MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicBase64PdfSource,
    MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicPlainTextSource,
    MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicContentBlockSource,
    MessageContentUnionMember1AnthropicRequestDocumentBlockSourceAnthropicUrlpdfSource,
]


class MessageContentUnionMember1AnthropicRequestDocumentBlockCitations(
    TypedDict, total=False
):
    enabled: bool


class MessageContentUnionMember1AnthropicRequestDocumentBlock(TypedDict, total=False):
    """
    Document content, either specified directly as base64 data, as text, or as a reference via a URL.
    """

    source: Required[MessageContentUnionMember1AnthropicRequestDocumentBlockSource]

    type: Required[Literal["document"]]

    cache_control: Optional[CacheControlEphemeralParam]
    """Create a cache control breakpoint at this content block."""

    citations: Optional[
        MessageContentUnionMember1AnthropicRequestDocumentBlockCitations
    ]

    context: Optional[str]

    title: Optional[str]


class MessageContentUnionMember1AnthropicRequestThinkingBlock(TypedDict, total=False):
    """A block specifying internal thinking by the model."""

    signature: Required[str]

    thinking: Required[str]

    type: Required[Literal["thinking"]]


class MessageContentUnionMember1AnthropicRequestRedactedThinkingBlock(
    TypedDict, total=False
):
    """A block specifying internal, redacted thinking by the model."""

    data: Required[str]

    type: Required[Literal["redacted_thinking"]]


class MessageContentUnionMember1AnthropicRequestToolUseBlock(TypedDict, total=False):
    """A block indicating a tool use by the model."""

    id: Required[str]

    input: Required[Dict[str, object]]

    name: Required[str]

    type: Required[Literal["tool_use"]]

    cache_control: Optional[CacheControlEphemeralParam]
    """Create a cache control breakpoint at this content block."""


class ContentAnthropicRequestDocumentBlockSourceAnthropicBase64PdfSource(
    TypedDict, total=False
):
    data: Required[
        Annotated[Union[str, Base64FileInput], PropertyInfo(format="base64")]
    ]

    media_type: Required[Literal["application/pdf"]]

    type: Required[Literal["base64"]]


set_pydantic_config(
    ContentAnthropicRequestDocumentBlockSourceAnthropicBase64PdfSource,
    {"arbitrary_types_allowed": True},
)


class ContentAnthropicRequestDocumentBlockSourceAnthropicPlainTextSource(
    TypedDict, total=False
):
    data: Required[str]

    media_type: Required[Literal["text/plain"]]

    type: Required[Literal["text"]]


ContentAnthropicRequestDocumentBlockSourceAnthropicContentBlockSourceContentContentBlockSourceContent: (
    TypeAlias
) = Union[
    RequestTextBlockParam, RequestImageBlockParam
]


class ContentAnthropicRequestDocumentBlockSourceAnthropicContentBlockSource(
    TypedDict, total=False
):
    content: Required[
        Union[
            str,
            Iterable[
                ContentAnthropicRequestDocumentBlockSourceAnthropicContentBlockSourceContentContentBlockSourceContent
            ],
        ]
    ]

    type: Required[Literal["content"]]


class ContentAnthropicRequestDocumentBlockSourceAnthropicUrlpdfSource(
    TypedDict, total=False
):
    type: Required[Literal["url"]]

    url: Required[str]


ContentAnthropicRequestDocumentBlockSource: TypeAlias = Union[
    ContentAnthropicRequestDocumentBlockSourceAnthropicBase64PdfSource,
    ContentAnthropicRequestDocumentBlockSourceAnthropicPlainTextSource,
    ContentAnthropicRequestDocumentBlockSourceAnthropicContentBlockSource,
    ContentAnthropicRequestDocumentBlockSourceAnthropicUrlpdfSource,
]


class ContentAnthropicRequestDocumentBlockCitations(TypedDict, total=False):
    enabled: bool


class ContentAnthropicRequestDocumentBlock(TypedDict, total=False):
    source: Required[ContentAnthropicRequestDocumentBlockSource]

    type: Required[Literal["document"]]

    cache_control: Optional[CacheControlEphemeralParam]
    """Create a cache control breakpoint at this content block."""

    citations: Optional[ContentAnthropicRequestDocumentBlockCitations]

    context: Optional[str]

    title: Optional[str]


Content: TypeAlias = Union[
    RequestTextBlockParam, RequestImageBlockParam, ContentAnthropicRequestDocumentBlock
]


class MessageContentUnionMember1AnthropicRequestToolResultBlock(TypedDict, total=False):
    """A block specifying the results of a tool use by the model."""

    tool_use_id: Required[str]

    type: Required[Literal["tool_result"]]

    cache_control: Optional[CacheControlEphemeralParam]
    """Create a cache control breakpoint at this content block."""

    content: Union[str, Iterable[Content]]

    is_error: bool


class Message(TypedDict, total=False):
    content: Required[
        Union[
            str,
            Iterable[
                Union[
                    RequestTextBlockParam,
                    RequestImageBlockParam,
                    MessageContentUnionMember1AnthropicRequestDocumentBlock,
                    MessageContentUnionMember1AnthropicRequestThinkingBlock,
                    MessageContentUnionMember1AnthropicRequestRedactedThinkingBlock,
                    MessageContentUnionMember1AnthropicRequestToolUseBlock,
                    MessageContentUnionMember1AnthropicRequestToolResultBlock,
                    ContentBlock,
                ]
            ],
        ]
    ]

    role: Required[Literal["user", "assistant"]]


class Metadata(TypedDict, total=False):
    """An object describing metadata about the request."""

    user_id: Optional[str]
    """An external identifier for the user who is associated with the request.

    This should be a uuid, hash value, or other opaque identifier. This id may be
    used to help detect abuse. Do not include any identifying information such as
    name, email address, or phone number.
    """


class OutputConfigFormat(TypedDict, total=False):
    """A schema to specify the model's output format in responses.

    See [structured outputs](/structured-responses/structured-output-grammar-based)
    """

    schema: Required[Dict[str, object]]
    """The JSON schema of the format"""

    type: Required[Literal["json_schema"]]


class OutputConfig(TypedDict, total=False):
    """Configuration options for the model's output, such as the output format."""

    effort: Optional[Literal["low", "medium", "high", "max"]]
    """All possible effort levels."""

    format: Optional[OutputConfigFormat]
    """A schema to specify the model's output format in responses.

    See [structured outputs](/structured-responses/structured-output-grammar-based)
    """


class ThinkingAnthropicThinkingConfigEnabled(TypedDict, total=False):
    budget_tokens: Required[int]
    """Determines how many tokens the model can use for its internal reasoning process.

    Larger budgets can enable more thorough analysis for complex problems, improving
    response quality.

    Must be ≥1024 and less than `max_tokens`.

    See [reasoning](/guides/reasoning) for details.
    """

    type: Required[Literal["enabled"]]


class ThinkingAnthropicThinkingConfigDisabled(TypedDict, total=False):
    type: Required[Literal["disabled"]]


class ThinkingAnthropicThinkingConfigAdaptive(TypedDict, total=False):
    """**Not supported yet.**"""

    type: Required[Literal["adaptive"]]


Thinking: TypeAlias = Union[
    ThinkingAnthropicThinkingConfigEnabled,
    ThinkingAnthropicThinkingConfigDisabled,
    ThinkingAnthropicThinkingConfigAdaptive,
]


class ToolChoiceAnthropicToolChoiceAuto(TypedDict, total=False):
    """The model will automatically decide whether to use tools."""

    type: Required[Literal["auto"]]

    disable_parallel_tool_use: bool
    """Whether to disable parallel tool use.

    Defaults to `false`. If set to `true`, the model will output at most one tool
    use.
    """


class ToolChoiceAnthropicToolChoiceAny(TypedDict, total=False):
    """The model will use any available tools."""

    type: Required[Literal["any"]]

    disable_parallel_tool_use: bool
    """Whether to disable parallel tool use.

    Defaults to `false`. If set to `true`, the model will output exactly one tool
    use.
    """


class ToolChoiceAnthropicToolChoiceTool(TypedDict, total=False):
    """The model will use the specified tool with `tool_choice.name`."""

    name: Required[str]
    """The name of the tool to use."""

    type: Required[Literal["tool"]]

    disable_parallel_tool_use: bool
    """Whether to disable parallel tool use.

    Defaults to `false`. If set to `true`, the model will output exactly one tool
    use.
    """


class ToolChoiceAnthropicToolChoiceNone(TypedDict, total=False):
    """The model will not be allowed to use tools."""

    type: Required[Literal["none"]]


ToolChoice: TypeAlias = Union[
    ToolChoiceAnthropicToolChoiceAuto,
    ToolChoiceAnthropicToolChoiceAny,
    ToolChoiceAnthropicToolChoiceTool,
    ToolChoiceAnthropicToolChoiceNone,
]


class ToolInputSchema(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """[JSON schema](https://json-schema.org/draft/2020-12) for this tool's input.

    This defines the shape of the `input` that your tool accepts and that the model will produce.
    """

    type: Required[Literal["object"]]

    properties: Optional[Dict[str, object]]

    required: Optional[SequenceNotStr[str]]


class Tool(TypedDict, total=False):
    input_schema: Required[ToolInputSchema]
    """[JSON schema](https://json-schema.org/draft/2020-12) for this tool's input.

    This defines the shape of the `input` that your tool accepts and that the model
    will produce.
    """

    name: Required[str]
    """Name of the tool.

    This is how the tool will be called by the model and in `tool_use` blocks.
    """

    description: str
    """Description of what this tool does.

    Tool descriptions should be as detailed as possible. The more information that
    the model has about what the tool is and how to use it, the better it will
    perform. You can use natural language descriptions to reinforce important
    aspects of the tool input JSON schema.
    """

    strict: bool
    """When true, guarantees schema validation on tool names and inputs"""

    type: Optional[Literal["custom"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional

import httpx

from ..types import message_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.message_create_response import MessageCreateResponse
from ..types.request_text_block_param import RequestTextBlockParam

__all__ = ["MessagesResource", "AsyncMessagesResource"]


class MessagesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MessagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return MessagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MessagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return MessagesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        messages: Iterable[message_create_params.Message],
        model: str,
        max_tokens: int | Omit = omit,
        metadata: message_create_params.Metadata | Omit = omit,
        output_config: message_create_params.OutputConfig | Omit = omit,
        raw_output: Optional[bool] | Omit = omit,
        stop_sequences: SequenceNotStr[str] | Omit = omit,
        stream: bool | Omit = omit,
        system: Union[str, Iterable[RequestTextBlockParam]] | Omit = omit,
        temperature: float | Omit = omit,
        thinking: message_create_params.Thinking | Omit = omit,
        tool_choice: message_create_params.ToolChoice | Omit = omit,
        tools: Iterable[message_create_params.Tool] | Omit = omit,
        top_k: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageCreateResponse:
        """
        **Anthropic-compatible endpoint.**

        Send a structured list of input messages with text and/or image content, and the
        model will generate the next message in the conversation.

        The Messages API can be used for either single queries or stateless multi-turn
        conversations.

        **Fireworks Quickstarts:**

        - [Serverless Quickstart](/getting-started/quickstart)
        - [Deployments Quickstart](/getting-started/ondemand-quickstart)

        Args:
          messages: Input messages.

              Models are trained to operate on alternating `user` and `assistant`
              conversational turns. When creating a new `Message`, you specify the prior
              conversational turns with the `messages` parameter, and the model then generates
              the next `Message` in the conversation. Consecutive `user` or `assistant` turns
              in your request will be combined into a single turn.

              Each input message must be an object with a `role` and `content`. You can
              specify a single `user`-role message, or you can include multiple `user` and
              `assistant` messages.

              If the final message uses the `assistant` role, the response content will
              continue immediately from the content in that message. This can be used to
              constrain part of the model's response.

              Example with a single `user` message:

              ```json
              [{ "role": "user", "content": "Hello" }]
              ```

              Example with multiple conversational turns:

              ```json
              [
                { "role": "user", "content": "Hello there." },
                {
                  "role": "assistant",
                  "content": "Hi, I'm here to help. How can I help you?"
                },
                { "role": "user", "content": "Can you explain LLMs in plain English?" }
              ]
              ```

              Example with a partially-filled response from the model:

              ```json
              [
                {
                  "role": "user",
                  "content": "What's the Greek name for Sun? (A) Sol (B) Helios (C) Sun"
                },
                { "role": "assistant", "content": "The best answer is (" }
              ]
              ```

              Each input message `content` may be either a single `string` or an array of
              content blocks, where each block has a specific `type`. Using a `string` for
              `content` is shorthand for an array of one content block of type `"text"`. The
              following input messages are equivalent:

              ```json
              { "role": "user", "content": "Hello" }
              ```

              ```json
              { "role": "user", "content": [{ "type": "text", "text": "Hello" }] }
              ```

              See [input examples](https://docs.claude.com/en/api/messages-examples).

              Note that if you want to include a
              [system prompt](/guides/querying-text-models), you can use the top-level
              `system` parameter — there is no `"system"` role for input messages in the
              Messages API.

              There is a limit of 100,000 messages in a single request.

          model: The model that will complete your prompt. See the
              [Fireworks Model Library](https://app.fireworks.ai/models) for available models.

          max_tokens: The maximum number of tokens to generate before stopping.

              Note that models may stop _before_ reaching this maximum. This parameter only
              specifies the absolute maximum number of tokens to generate.

              Different models have different maximum values for this parameter. See
              [models](https://app.fireworks.ai/models) for details.

          metadata: An object describing metadata about the request.

          output_config: Configuration options for the model's output, such as the output format.

          raw_output: Return raw output from the model.

          stop_sequences: Custom text sequences that will cause the model to stop generating.

              Models will normally stop when they have naturally completed their turn, which
              will result in a response `stop_reason` of `"end_turn"`.

              If you want the model to stop generating when it encounters custom strings of
              text, you can use the `stop_sequences` parameter. If the model encounters one of
              the custom sequences, the response `stop_reason` value will be `"stop_sequence"`
              and the response `stop_sequence` value will contain the matched stop sequence.

          stream: Whether to incrementally stream the response using server-sent events.

              See [streaming](/guides/querying-text-models) for details.

          system: System prompt.

              A system prompt is a way of providing context and instructions to the model,
              such as specifying a particular goal or role. See the
              [guide to system prompts](/guides/querying-text-models).

          temperature: Amount of randomness injected into the response.

              Defaults to `1.0`. Ranges from `0.0` to `1.0`. Use `temperature` closer to `0.0`
              for analytical / multiple choice, and closer to `1.0` for creative and
              generative tasks.

              Note that even with `temperature` of `0.0`, the results will not be fully
              deterministic.

          thinking: Configuration for enabling the model's extended thinking.

              When enabled, responses include `thinking` content blocks showing the model's
              thinking process before the final answer. Requires a minimum budget of 1,024
              tokens and counts towards your `max_tokens` limit.

              See [reasoning](/guides/reasoning) for details.

              **Note:** The `adaptive` thinking type is not supported yet.

          tool_choice: How the model should use the provided tools. The model can use a specific tool,
              any available tool, decide by itself, or not use tools at all.

          tools: Definitions of tools that the model may use.

              If you include `tools` in your API request, the model may return `tool_use`
              content blocks that represent the model's use of those tools. You can then run
              those tools using the tool input generated by the model and then optionally
              return results back to the model using `tool_result` content blocks.

              Each tool definition includes:

              - `name`: Name of the tool.
              - `description`: Optional, but strongly-recommended description of the tool.
              - `input_schema`: [JSON schema](https://json-schema.org/draft/2020-12) for the
                tool `input` shape that the model will produce in `tool_use` output content
                blocks.

              For example, if you defined `tools` as:

              ```json
              [
                {
                  "name": "get_stock_price",
                  "description": "Get the current stock price for a given ticker symbol.",
                  "input_schema": {
                    "type": "object",
                    "properties": {
                      "ticker": {
                        "type": "string",
                        "description": "The stock ticker symbol, e.g. AAPL for Apple Inc."
                      }
                    },
                    "required": ["ticker"]
                  }
                }
              ]
              ```

              And then asked the model "What's the S&P 500 at today?", the model might produce
              `tool_use` content blocks in the response like this:

              ```json
              [
                {
                  "type": "tool_use",
                  "id": "toolu_01D7FLrfh4GYq7yT1ULFeyMV",
                  "name": "get_stock_price",
                  "input": { "ticker": "^GSPC" }
                }
              ]
              ```

              You might then run your `get_stock_price` tool with `{"ticker": "^GSPC"}` as an
              input, and return the following back to the model in a subsequent `user`
              message:

              ```json
              [
                {
                  "type": "tool_result",
                  "tool_use_id": "toolu_01D7FLrfh4GYq7yT1ULFeyMV",
                  "content": "259.75 USD"
                }
              ]
              ```

              Tools can be used for workflows that include running client-side tools and
              functions, or more generally whenever you want the model to produce a particular
              JSON structure of output.

              See the [guide](/guides/function-calling) for more details.

          top_k: Only sample from the top K options for each subsequent token.

              Used to remove "long tail" low probability responses.
              [Learn more technical details here](https://towardsdatascience.com/how-to-sample-from-language-models-682bceb97277).

              Recommended for advanced use cases only. You usually only need to use
              `temperature`.

          top_p: Use nucleus sampling.

              In nucleus sampling, we compute the cumulative distribution over all the options
              for each subsequent token in decreasing probability order and cut it off once it
              reaches a particular probability specified by `top_p`. You should either alter
              `temperature` or `top_p`, but not both.

              Recommended for advanced use cases only. You usually only need to use
              `temperature`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/messages" if self._client._base_url_overridden else "https://api.fireworks.ai/inference/v1/messages",
            body=maybe_transform(
                {
                    "messages": messages,
                    "model": model,
                    "max_tokens": max_tokens,
                    "metadata": metadata,
                    "output_config": output_config,
                    "raw_output": raw_output,
                    "stop_sequences": stop_sequences,
                    "stream": stream,
                    "system": system,
                    "temperature": temperature,
                    "thinking": thinking,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_k": top_k,
                    "top_p": top_p,
                },
                message_create_params.MessageCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageCreateResponse,
        )


class AsyncMessagesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMessagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncMessagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMessagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return AsyncMessagesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        messages: Iterable[message_create_params.Message],
        model: str,
        max_tokens: int | Omit = omit,
        metadata: message_create_params.Metadata | Omit = omit,
        output_config: message_create_params.OutputConfig | Omit = omit,
        raw_output: Optional[bool] | Omit = omit,
        stop_sequences: SequenceNotStr[str] | Omit = omit,
        stream: bool | Omit = omit,
        system: Union[str, Iterable[RequestTextBlockParam]] | Omit = omit,
        temperature: float | Omit = omit,
        thinking: message_create_params.Thinking | Omit = omit,
        tool_choice: message_create_params.ToolChoice | Omit = omit,
        tools: Iterable[message_create_params.Tool] | Omit = omit,
        top_k: int | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageCreateResponse:
        """
        **Anthropic-compatible endpoint.**

        Send a structured list of input messages with text and/or image content, and the
        model will generate the next message in the conversation.

        The Messages API can be used for either single queries or stateless multi-turn
        conversations.

        **Fireworks Quickstarts:**

        - [Serverless Quickstart](/getting-started/quickstart)
        - [Deployments Quickstart](/getting-started/ondemand-quickstart)

        Args:
          messages: Input messages.

              Models are trained to operate on alternating `user` and `assistant`
              conversational turns. When creating a new `Message`, you specify the prior
              conversational turns with the `messages` parameter, and the model then generates
              the next `Message` in the conversation. Consecutive `user` or `assistant` turns
              in your request will be combined into a single turn.

              Each input message must be an object with a `role` and `content`. You can
              specify a single `user`-role message, or you can include multiple `user` and
              `assistant` messages.

              If the final message uses the `assistant` role, the response content will
              continue immediately from the content in that message. This can be used to
              constrain part of the model's response.

              Example with a single `user` message:

              ```json
              [{ "role": "user", "content": "Hello" }]
              ```

              Example with multiple conversational turns:

              ```json
              [
                { "role": "user", "content": "Hello there." },
                {
                  "role": "assistant",
                  "content": "Hi, I'm here to help. How can I help you?"
                },
                { "role": "user", "content": "Can you explain LLMs in plain English?" }
              ]
              ```

              Example with a partially-filled response from the model:

              ```json
              [
                {
                  "role": "user",
                  "content": "What's the Greek name for Sun? (A) Sol (B) Helios (C) Sun"
                },
                { "role": "assistant", "content": "The best answer is (" }
              ]
              ```

              Each input message `content` may be either a single `string` or an array of
              content blocks, where each block has a specific `type`. Using a `string` for
              `content` is shorthand for an array of one content block of type `"text"`. The
              following input messages are equivalent:

              ```json
              { "role": "user", "content": "Hello" }
              ```

              ```json
              { "role": "user", "content": [{ "type": "text", "text": "Hello" }] }
              ```

              See [input examples](https://docs.claude.com/en/api/messages-examples).

              Note that if you want to include a
              [system prompt](/guides/querying-text-models), you can use the top-level
              `system` parameter — there is no `"system"` role for input messages in the
              Messages API.

              There is a limit of 100,000 messages in a single request.

          model: The model that will complete your prompt. See the
              [Fireworks Model Library](https://app.fireworks.ai/models) for available models.

          max_tokens: The maximum number of tokens to generate before stopping.

              Note that models may stop _before_ reaching this maximum. This parameter only
              specifies the absolute maximum number of tokens to generate.

              Different models have different maximum values for this parameter. See
              [models](https://app.fireworks.ai/models) for details.

          metadata: An object describing metadata about the request.

          output_config: Configuration options for the model's output, such as the output format.

          raw_output: Return raw output from the model.

          stop_sequences: Custom text sequences that will cause the model to stop generating.

              Models will normally stop when they have naturally completed their turn, which
              will result in a response `stop_reason` of `"end_turn"`.

              If you want the model to stop generating when it encounters custom strings of
              text, you can use the `stop_sequences` parameter. If the model encounters one of
              the custom sequences, the response `stop_reason` value will be `"stop_sequence"`
              and the response `stop_sequence` value will contain the matched stop sequence.

          stream: Whether to incrementally stream the response using server-sent events.

              See [streaming](/guides/querying-text-models) for details.

          system: System prompt.

              A system prompt is a way of providing context and instructions to the model,
              such as specifying a particular goal or role. See the
              [guide to system prompts](/guides/querying-text-models).

          temperature: Amount of randomness injected into the response.

              Defaults to `1.0`. Ranges from `0.0` to `1.0`. Use `temperature` closer to `0.0`
              for analytical / multiple choice, and closer to `1.0` for creative and
              generative tasks.

              Note that even with `temperature` of `0.0`, the results will not be fully
              deterministic.

          thinking: Configuration for enabling the model's extended thinking.

              When enabled, responses include `thinking` content blocks showing the model's
              thinking process before the final answer. Requires a minimum budget of 1,024
              tokens and counts towards your `max_tokens` limit.

              See [reasoning](/guides/reasoning) for details.

              **Note:** The `adaptive` thinking type is not supported yet.

          tool_choice: How the model should use the provided tools. The model can use a specific tool,
              any available tool, decide by itself, or not use tools at all.

          tools: Definitions of tools that the model may use.

              If you include `tools` in your API request, the model may return `tool_use`
              content blocks that represent the model's use of those tools. You can then run
              those tools using the tool input generated by the model and then optionally
              return results back to the model using `tool_result` content blocks.

              Each tool definition includes:

              - `name`: Name of the tool.
              - `description`: Optional, but strongly-recommended description of the tool.
              - `input_schema`: [JSON schema](https://json-schema.org/draft/2020-12) for the
                tool `input` shape that the model will produce in `tool_use` output content
                blocks.

              For example, if you defined `tools` as:

              ```json
              [
                {
                  "name": "get_stock_price",
                  "description": "Get the current stock price for a given ticker symbol.",
                  "input_schema": {
                    "type": "object",
                    "properties": {
                      "ticker": {
                        "type": "string",
                        "description": "The stock ticker symbol, e.g. AAPL for Apple Inc."
                      }
                    },
                    "required": ["ticker"]
                  }
                }
              ]
              ```

              And then asked the model "What's the S&P 500 at today?", the model might produce
              `tool_use` content blocks in the response like this:

              ```json
              [
                {
                  "type": "tool_use",
                  "id": "toolu_01D7FLrfh4GYq7yT1ULFeyMV",
                  "name": "get_stock_price",
                  "input": { "ticker": "^GSPC" }
                }
              ]
              ```

              You might then run your `get_stock_price` tool with `{"ticker": "^GSPC"}` as an
              input, and return the following back to the model in a subsequent `user`
              message:

              ```json
              [
                {
                  "type": "tool_result",
                  "tool_use_id": "toolu_01D7FLrfh4GYq7yT1ULFeyMV",
                  "content": "259.75 USD"
                }
              ]
              ```

              Tools can be used for workflows that include running client-side tools and
              functions, or more generally whenever you want the model to produce a particular
              JSON structure of output.

              See the [guide](/guides/function-calling) for more details.

          top_k: Only sample from the top K options for each subsequent token.

              Used to remove "long tail" low probability responses.
              [Learn more technical details here](https://towardsdatascience.com/how-to-sample-from-language-models-682bceb97277).

              Recommended for advanced use cases only. You usually only need to use
              `temperature`.

          top_p: Use nucleus sampling.

              In nucleus sampling, we compute the cumulative distribution over all the options
              for each subsequent token in decreasing probability order and cut it off once it
              reaches a particular probability specified by `top_p`. You should either alter
              `temperature` or `top_p`, but not both.

              Recommended for advanced use cases only. You usually only need to use
              `temperature`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/messages" if self._client._base_url_overridden else "https://api.fireworks.ai/inference/v1/messages",
            body=await async_maybe_transform(
                {
                    "messages": messages,
                    "model": model,
                    "max_tokens": max_tokens,
                    "metadata": metadata,
                    "output_config": output_config,
                    "raw_output": raw_output,
                    "stop_sequences": stop_sequences,
                    "stream": stream,
                    "system": system,
                    "temperature": temperature,
                    "thinking": thinking,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_k": top_k,
                    "top_p": top_p,
                },
                message_create_params.MessageCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageCreateResponse,
        )


class MessagesResourceWithRawResponse:
    def __init__(self, messages: MessagesResource) -> None:
        self._messages = messages

        self.create = to_raw_response_wrapper(
            messages.create,
        )


class AsyncMessagesResourceWithRawResponse:
    def __init__(self, messages: AsyncMessagesResource) -> None:
        self._messages = messages

        self.create = async_to_raw_response_wrapper(
            messages.create,
        )


class MessagesResourceWithStreamingResponse:
    def __init__(self, messages: MessagesResource) -> None:
        self._messages = messages

        self.create = to_streamed_response_wrapper(
            messages.create,
        )


class AsyncMessagesResourceWithStreamingResponse:
    def __init__(self, messages: AsyncMessagesResource) -> None:
        self._messages = messages

        self.create = async_to_streamed_response_wrapper(
            messages.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.status import Status

__all__ = ["Account"]


class Account(BaseModel):
    email: str
    """The primary email for the account.

    This is used for billing invoices and account notifications.
    """

    account_type: Optional[Literal["ACCOUNT_TYPE_UNSPECIFIED", "ENTERPRISE"]] = FieldInfo(
        alias="accountType", default=None
    )
    """The type of the account."""

    create_time: Optional[datetime] = FieldInfo(alias="createTime", default=None)
    """The creation time of the account."""

    display_name: Optional[str] = FieldInfo(alias="displayName", default=None)
    """Human-readable display name of the account.

    e.g. "My Account" Must be fewer than 64 characters long.
    """

    name: Optional[str] = None

    state: Optional[Literal["STATE_UNSPECIFIED", "CREATING", "READY", "UPDATING", "DELETING"]] = None
    """The state of the account."""

    status: Optional[Status] = None
    """Contains information about the account status."""

    suspend_state: Optional[
        Literal[
            "UNSUSPENDED", "FAILED_PAYMENTS", "CREDIT_DEPLETED", "MONTHLY_SPEND_LIMIT_EXCEEDED", "BLOCKED_BY_ABUSE_RULE"
        ]
    ] = FieldInfo(alias="suspendState", default=None)

    update_time: Optional[datetime] = FieldInfo(alias="updateTime", default=None)
    """The update time for the account."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ResponseRedactedThinkingBlock"]


class ResponseRedactedThinkingBlock(BaseModel):
    data: str

    type: Literal["redacted_thinking"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import builtins
from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel
from ..shared.log_probs import LogProbs
from ..shared.raw_output import RawOutput
from ..shared.usage_info import UsageInfo
from ..shared.new_log_probs import NewLogProbs
from ..shared.chat_completion_message_tool_call import ChatCompletionMessageToolCall

__all__ = ["ChatCompletionChunk", "Choice", "ChoiceDelta", "ChoiceLogprobs"]


class ChoiceDelta(BaseModel):
    """The message delta"""

    content: Optional[str] = None
    """The contents of the chunk message"""

    reasoning_content: Optional[str] = None
    """The reasoning or thinking process generated by the model.

    This field is only available for certain reasoning models (GLM 4.5, GLM 4.5 Air,
    GPT OSS 120B, GPT OSS 20B) and contains the model's internal reasoning that
    would otherwise appear in `<think></think>` tags within the content field.
    """

    role: Optional[str] = None
    """The role of the author of this message"""

    tool_calls: Optional[List[ChatCompletionMessageToolCall]] = None


ChoiceLogprobs: TypeAlias = Union[LogProbs, NewLogProbs, None]


class Choice(BaseModel):
    """A streamed chat completion choice."""

    delta: ChoiceDelta
    """The message delta"""

    index: int
    """The index of the chat completion choice"""

    finish_reason: Optional[Literal["stop", "length", "function_call", "tool_calls"]] = None
    """The reason the model stopped generating tokens.

    This will be "stop" if the model hit a natural stop point or a provided stop
    sequence, or "length" if the maximum number of tokens specified in the request
    was reached
    """

    logprobs: Optional[ChoiceLogprobs] = None
    """Legacy log probabilities format"""

    prompt_token_ids: Optional[List[int]] = None
    """Token IDs for the prompt (when return_token_ids=true)"""

    raw_output: Optional[RawOutput] = None
    """
    Extension of OpenAI that returns low-level interaction of what the model sees,
    including the formatted prompt and function calls
    """

    token_ids: Optional[List[int]] = None
    """Token IDs for this chunk (when return_token_ids=true)"""


class ChatCompletionChunk(BaseModel):
    """The streamed response message from a /v1/chat/completions call."""

    id: str
    """A unique identifier of the response"""

    choices: List[Choice]
    """The list of streamed chat completion choices"""

    created: int
    """The Unix time in seconds when the response was generated"""

    model: str
    """The model used for the chat completion"""

    object: Optional[str] = None
    """The object type, which is always "chat.completion.chunk" """

    perf_metrics: Optional[Dict[str, builtins.object]] = None
    """See parameter [perf_metrics_in_response](#body-perf-metrics-in-response)"""

    prompt_token_ids: Optional[List[int]] = None
    """Token IDs for the prompt (when return_token_ids=true, sent in first chunk)"""

    usage: Optional[UsageInfo] = None
    """Usage statistics."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .content_block import ContentBlock, ContentBlock as ContentBlock

__all__ = ["MessageCreateResponse", "RawOutput"]


class RawOutput(BaseModel):
    """
    Fireworks extension that returns low-level details of what the model sees, including the formatted prompt and function calls.
    """

    completion: str
    """Raw completion produced by the model before any tool calls are parsed"""

    prompt_fragments: List[Union[str, int]]
    """
    Pieces of the prompt (like individual messages) before truncation and
    concatenation. Depending on prompt_truncate_len some of the messages might be
    dropped. Contains a mix of strings to be tokenized and individual tokens (if
    dictated by the conversation template)
    """

    prompt_token_ids: List[int]
    """Fully processed prompt as seen by the model"""

    completion_token_ids: Optional[List[int]] = None
    """Token IDs for the raw completion"""

    grammar: Optional[str] = None
    """
    Grammar used for constrained decoding, can be either user provided (directly or
    JSON schema) or inferred by the chat template
    """

    images: Optional[List[str]] = None
    """Images in the prompt"""


class MessageCreateResponse(BaseModel):
    id: str
    """Unique object identifier.

    The format and length of IDs may change over time.
    """

    content: List[ContentBlock]
    """Content generated by the model.

    This is an array of content blocks, each of which has a `type` that determines
    its shape.

    Example:

    ```json
    [{ "type": "text", "text": "Hi, I'm here to help." }]
    ```

    If the request input `messages` ended with an `assistant` turn, then the
    response `content` will continue directly from that last turn. You can use this
    to constrain the model's output.

    For example, if the input `messages` were:

    ```json
    [
      {
        "role": "user",
        "content": "What's the Greek name for Sun? (A) Sol (B) Helios (C) Sun"
      },
      { "role": "assistant", "content": "The best answer is (" }
    ]
    ```

    Then the response `content` might be:

    ```json
    [{ "type": "text", "text": "B)" }]
    ```
    """

    model: str
    """The model that will complete your prompt.

    See the [Fireworks Model Library](https://app.fireworks.ai/models) for available
    models.
    """

    role: Literal["assistant"]
    """Conversational role of the generated message.

    This will always be `"assistant"`.
    """

    stop_reason: Optional[Literal["end_turn", "max_tokens", "stop_sequence", "tool_use", "pause_turn", "refusal"]] = (
        None
    )
    """The reason that the model stopped.

    This may be one the following values:

    - `"end_turn"`: the model reached a natural stopping point
    - `"max_tokens"`: the model exceeded the requested `max_tokens` or the model's
      maximum
    - `"stop_sequence"`: one of your provided custom `stop_sequences` was generated
    - `"tool_use"`: the model invoked one or more tools
    - `"pause_turn"`: the model paused a long-running turn. You may provide the
      response back as-is in a subsequent request to let the model continue.
    - `"refusal"`: when streaming classifiers intervene to handle potential policy
      violations

    In non-streaming mode this value is always non-null. In streaming mode, it is
    null in the `message_start` event and non-null otherwise.
    """

    stop_sequence: Optional[str] = None
    """Which custom stop sequence was generated, if any.

    This value will be a non-null string if one of your custom stop sequences was
    generated.
    """

    type: Literal["message"]
    """Object type.

    For Messages, this is always `"message"`.
    """

    raw_output: Optional[RawOutput] = None
    """
    Fireworks extension that returns low-level details of what the model sees,
    including the formatted prompt and function calls.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import builtins
from typing import Dict, List, Optional

from .._models import BaseModel
from .shared.choice import Choice
from .shared.usage_info import UsageInfo

__all__ = ["CompletionCreateResponse"]


class CompletionCreateResponse(BaseModel):
    """The response message from a /v1/completions call."""

    id: str
    """A unique identifier of the response"""

    choices: List[Choice]
    """The list of generated completion choices"""

    created: int
    """The Unix time in seconds when the response was generated"""

    model: str
    """The model used for the completion"""

    usage: UsageInfo
    """Usage statistics for the completion"""

    object: Optional[str] = None
    """The object type, which is always "text_completion" """

    perf_metrics: Optional[Dict[str, builtins.object]] = None
    """See parameter [perf_metrics_in_response](#body-perf-metrics-in-response)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .deployment_shape import DeploymentShape

__all__ = ["DeploymentShapeVersion"]


class DeploymentShapeVersion(BaseModel):
    create_time: Optional[datetime] = FieldInfo(alias="createTime", default=None)
    """The creation time of the deployment shape version.

    Lists will be ordered by this field.
    """

    latest_validated: Optional[bool] = FieldInfo(alias="latestValidated", default=None)
    """
    If true, this version is the latest validated version. Only one version of the
    shape can be the latest validated version.
    """

    name: Optional[str] = None

    public: Optional[bool] = None
    """If true, this version will be publicly readable."""

    snapshot: Optional[DeploymentShape] = None
    """Full snapshot of the Deployment Shape at this version."""

    validated: Optional[bool] = None
    """If true, this version has been validated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from fireworks import Fireworks, AsyncFireworks
from tests.utils import assert_matches_type
from fireworks.types.chat import CompletionCreateResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCompletions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_overload_1(self, client: Fireworks) -> None:
        completion = client.chat.completions.create(
            messages=[{"role": "role"}],
            model="model",
        )
        assert_matches_type(CompletionCreateResponse, completion, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params_overload_1(self, client: Fireworks) -> None:
        completion = client.chat.completions.create(
            messages=[
                {
                    "role": "role",
                    "content": "string",
                    "reasoning_content": "reasoning_content",
                    "tool_call_id": "tool_call_id",
                    "tool_calls": [
                        {
                            "function": {
                                "arguments": "string",
                                "name": "name",
                            },
                            "id": "id",
                            "type": "type",
                        }
                    ],
                }
            ],
            model="model",
            context_length_exceeded_behavior="error",
            echo=True,
            echo_last=0,
            frequency_penalty=0,
            function_call="auto",
            functions=[
                {
                    "name": "name",
                    "description": "description",
                    "parameters": {"foo": "bar"},
                    "strict": True,
                }
            ],
            ignore_eos=True,
            logit_bias={"foo": 0},
            logprobs=0,
            max_completion_tokens=0,
            max_tokens=0,
            metadata={"foo": "string"},
            min_p=0,
            mirostat_lr=0,
            mirostat_target=0,
            n=0,
            parallel_tool_calls=True,
            perf_metrics_in_response=True,
            prediction={
                "content": "string",
                "type": "content",
            },
            presence_penalty=0,
            prompt_cache_isolation_key="prompt_cache_isolation_key",
            prompt_truncate_len=0,
            raw_output=True,
            reasoning_effort="low",
            reasoning_history="disabled",
            repetition_penalty=0,
            response_format={
                "type": "json_object",
                "grammar": "grammar",
                "json_schema": "string",
                "schema": "string",
            },
            return_token_ids=True,
            seed=0,
            speculation="string",
            stop="string",
            stream=False,
            temperature=0,
            thinking={
                "type": "enabled",
                "budget_tokens": 0,
            },
            tool_choice="auto",
            tools=[
                {
                    "type": "function",
                    "function": {
                        "name": "name",
                        "description": "description",
                        "parameters": {"foo": "bar"},
                        "strict": True,
                    },
                }
            ],
            top_k=0,
            top_logprobs=0,
            top_p=0,
            typical_p=0,
            user="user",
        )
        assert_matches_type(CompletionCreateResponse, completion, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create_overload_1(self, client: Fireworks) -> None:
        response = client.chat.completions.with_raw_response.create(
            messages=[{"role": "role"}],
            model="model",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        completion = response.parse()
        assert_matches_type(CompletionCreateResponse, completion, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create_overload_1(self, client: Fireworks) -> None:
        with client.chat.completions.with_streaming_response.create(
            messages=[{"role": "role"}],
            model="model",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            completion = response.parse()
            assert_matches_type(CompletionCreateResponse, completion, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_overload_2(self, client: Fireworks) -> None:
        completion_stream = client.chat.completions.create(
            messages=[{"role": "role"}],
            model="model",
            stream=True,
        )
        completion_stream.response.close()

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params_overload_2(self, client: Fireworks) -> None:
        completion_stream = client.chat.completions.create(
            messages=[
                {
                    "role": "role",
                    "content": "string",
                    "reasoning_content": "reasoning_content",
                    "tool_call_id": "tool_call_id",
                    "tool_calls": [
                        {
                            "function": {
                                "arguments": "string",
                                "name": "name",
                            },
                            "id": "id",
                            "type": "type",
                        }
                    ],
                }
            ],
            model="model",
            stream=True,
            context_length_exceeded_behavior="error",
            echo=True,
            echo_last=0,
            frequency_penalty=0,
            function_call="auto",
            functions=[
                {
                    "name": "name",
                    "description": "description",
                    "parameters": {"foo": "bar"},
                    "strict": True,
                }
            ],
            ignore_eos=True,
            logit_bias={"foo": 0},
            logprobs=0,
            max_completion_tokens=0,
            max_tokens=0,
            metadata={"foo": "string"},
            min_p=0,
            mirostat_lr=0,
            mirostat_target=0,
            n=0,
            parallel_tool_calls=True,
            perf_metrics_in_response=True,
            prediction={
                "content": "string",
                "type": "content",
            },
            presence_penalty=0,
            prompt_cache_isolation_key="prompt_cache_isolation_key",
            prompt_truncate_len=0,
            raw_output=True,
            reasoning_effort="low",
            reasoning_history="disabled",
            repetition_penalty=0,
            response_format={
                "type": "json_object",
                "grammar": "grammar",
                "json_schema": "string",
                "schema": "string",
            },
            return_token_ids=True,
            seed=0,
            speculation="string",
            stop="string",
            temperature=0,
            thinking={
                "type": "enabled",
                "budget_tokens": 0,
            },
            tool_choice="auto",
            tools=[
                {
                    "type": "function",
                    "function": {
                        "name": "name",
                        "description": "description",
                        "parameters": {"foo": "bar"},
                        "strict": True,
                    },
                }
            ],
            top_k=0,
            top_logprobs=0,
            top_p=0,
            typical_p=0,
            user="user",
        )
        completion_stream.response.close()

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create_overload_2(self, client: Fireworks) -> None:
        response = client.chat.completions.with_raw_response.create(
            messages=[{"role": "role"}],
            model="model",
            stream=True,
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        stream.close()

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create_overload_2(self, client: Fireworks) -> None:
        with client.chat.completions.with_streaming_response.create(
            messages=[{"role": "role"}],
            model="model",
            stream=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = response.parse()
            stream.close()

        assert cast(Any, response.is_closed) is True


class TestAsyncCompletions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_overload_1(self, async_client: AsyncFireworks) -> None:
        completion = await async_client.chat.completions.create(
            messages=[{"role": "role"}],
            model="model",
        )
        assert_matches_type(CompletionCreateResponse, completion, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params_overload_1(self, async_client: AsyncFireworks) -> None:
        completion = await async_client.chat.completions.create(
            messages=[
                {
                    "role": "role",
                    "content": "string",
                    "reasoning_content": "reasoning_content",
                    "tool_call_id": "tool_call_id",
                    "tool_calls": [
                        {
                            "function": {
                                "arguments": "string",
                                "name": "name",
                            },
                            "id": "id",
                            "type": "type",
                        }
                    ],
                }
            ],
            model="model",
            context_length_exceeded_behavior="error",
            echo=True,
            echo_last=0,
            frequency_penalty=0,
            function_call="auto",
            functions=[
                {
                    "name": "name",
                    "description": "description",
                    "parameters": {"foo": "bar"},
                    "strict": True,
                }
            ],
            ignore_eos=True,
            logit_bias={"foo": 0},
            logprobs=0,
            max_completion_tokens=0,
            max_tokens=0,
            metadata={"foo": "string"},
            min_p=0,
            mirostat_lr=0,
            mirostat_target=0,
            n=0,
            parallel_tool_calls=True,
            perf_metrics_in_response=True,
            prediction={
                "content": "string",
                "type": "content",
            },
            presence_penalty=0,
            prompt_cache_isolation_key="prompt_cache_isolation_key",
            prompt_truncate_len=0,
            raw_output=True,
            reasoning_effort="low",
            reasoning_history="disabled",
            repetition_penalty=0,
            response_format={
                "type": "json_object",
                "grammar": "grammar",
                "json_schema": "string",
                "schema": "string",
            },
            return_token_ids=True,
            seed=0,
            speculation="string",
            stop="string",
            stream=False,
            temperature=0,
            thinking={
                "type": "enabled",
                "budget_tokens": 0,
            },
            tool_choice="auto",
            tools=[
                {
                    "type": "function",
                    "function": {
                        "name": "name",
                        "description": "description",
                        "parameters": {"foo": "bar"},
                        "strict": True,
                    },
                }
            ],
            top_k=0,
            top_logprobs=0,
            top_p=0,
            typical_p=0,
            user="user",
        )
        assert_matches_type(CompletionCreateResponse, completion, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create_overload_1(self, async_client: AsyncFireworks) -> None:
        response = await async_client.chat.completions.with_raw_response.create(
            messages=[{"role": "role"}],
            model="model",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        completion = await response.parse()
        assert_matches_type(CompletionCreateResponse, completion, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create_overload_1(self, async_client: AsyncFireworks) -> None:
        async with async_client.chat.completions.with_streaming_response.create(
            messages=[{"role": "role"}],
            model="model",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            completion = await response.parse()
            assert_matches_type(CompletionCreateResponse, completion, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_overload_2(self, async_client: AsyncFireworks) -> None:
        completion_stream = await async_client.chat.completions.create(
            messages=[{"role": "role"}],
            model="model",
            stream=True,
        )
        await completion_stream.response.aclose()

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params_overload_2(self, async_client: AsyncFireworks) -> None:
        completion_stream = await async_client.chat.completions.create(
            messages=[
                {
                    "role": "role",
                    "content": "string",
                    "reasoning_content": "reasoning_content",
                    "tool_call_id": "tool_call_id",
                    "tool_calls": [
                        {
                            "function": {
                                "arguments": "string",
                                "name": "name",
                            },
                            "id": "id",
                            "type": "type",
                        }
                    ],
                }
            ],
            model="model",
            stream=True,
            context_length_exceeded_behavior="error",
            echo=True,
            echo_last=0,
            frequency_penalty=0,
            function_call="auto",
            functions=[
                {
                    "name": "name",
                    "description": "description",
                    "parameters": {"foo": "bar"},
                    "strict": True,
                }
            ],
            ignore_eos=True,
            logit_bias={"foo": 0},
            logprobs=0,
            max_completion_tokens=0,
            max_tokens=0,
            metadata={"foo": "string"},
            min_p=0,
            mirostat_lr=0,
            mirostat_target=0,
            n=0,
            parallel_tool_calls=True,
            perf_metrics_in_response=True,
            prediction={
                "content": "string",
                "type": "content",
            },
            presence_penalty=0,
            prompt_cache_isolation_key="prompt_cache_isolation_key",
            prompt_truncate_len=0,
            raw_output=True,
            reasoning_effort="low",
            reasoning_history="disabled",
            repetition_penalty=0,
            response_format={
                "type": "json_object",
                "grammar": "grammar",
                "json_schema": "string",
                "schema": "string",
            },
            return_token_ids=True,
            seed=0,
            speculation="string",
            stop="string",
            temperature=0,
            thinking={
                "type": "enabled",
                "budget_tokens": 0,
            },
            tool_choice="auto",
            tools=[
                {
                    "type": "function",
                    "function": {
                        "name": "name",
                        "description": "description",
                        "parameters": {"foo": "bar"},
                        "strict": True,
                    },
                }
            ],
            top_k=0,
            top_logprobs=0,
            top_p=0,
            typical_p=0,
            user="user",
        )
        await completion_stream.response.aclose()

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create_overload_2(self, async_client: AsyncFireworks) -> None:
        response = await async_client.chat.completions.with_raw_response.create(
            messages=[{"role": "role"}],
            model="model",
            stream=True,
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = await response.parse()
        await stream.close()

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create_overload_2(self, async_client: AsyncFireworks) -> None:
        async with async_client.chat.completions.with_streaming_response.create(
            messages=[{"role": "role"}],
            model="model",
            stream=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = await response.parse()
            await stream.close()

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, FireworksError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import (
        chat,
        lora,
        users,
        models,
        secrets,
        accounts,
        api_keys,
        datasets,
        dpo_jobs,
        messages,
        evaluators,
        completions,
        deployments,
        evaluation_jobs,
        deployment_shapes,
        batch_inference_jobs,
        deployment_shape_versions,
        supervised_fine_tuning_jobs,
        reinforcement_fine_tuning_jobs,
        reinforcement_fine_tuning_steps,
    )
    from .resources.lora import LoraResource, AsyncLoraResource
    from .resources.users import UsersResource, AsyncUsersResource
    from .resources.models import ModelsResource, AsyncModelsResource
    from .resources.secrets import SecretsResource, AsyncSecretsResource
    from .resources.accounts import AccountsResource, AsyncAccountsResource
    from .resources.api_keys import APIKeysResource, AsyncAPIKeysResource
    from .resources.datasets import DatasetsResource, AsyncDatasetsResource
    from .resources.dpo_jobs import DpoJobsResource, AsyncDpoJobsResource
    from .resources.messages import MessagesResource, AsyncMessagesResource
    from .resources.chat.chat import ChatResource, AsyncChatResource
    from .resources.evaluators import EvaluatorsResource, AsyncEvaluatorsResource
    from .resources.completions import CompletionsResource, AsyncCompletionsResource
    from .resources.deployments import DeploymentsResource, AsyncDeploymentsResource
    from .resources.evaluation_jobs import EvaluationJobsResource, AsyncEvaluationJobsResource
    from .resources.deployment_shapes import DeploymentShapesResource, AsyncDeploymentShapesResource
    from .resources.batch_inference_jobs import BatchInferenceJobsResource, AsyncBatchInferenceJobsResource
    from .resources.deployment_shape_versions import (
        DeploymentShapeVersionsResource,
        AsyncDeploymentShapeVersionsResource,
    )
    from .resources.supervised_fine_tuning_jobs import (
        SupervisedFineTuningJobsResource,
        AsyncSupervisedFineTuningJobsResource,
    )
    from .resources.reinforcement_fine_tuning_jobs import (
        ReinforcementFineTuningJobsResource,
        AsyncReinforcementFineTuningJobsResource,
    )
    from .resources.reinforcement_fine_tuning_steps import (
        ReinforcementFineTuningStepsResource,
        AsyncReinforcementFineTuningStepsResource,
    )

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Fireworks",
    "AsyncFireworks",
    "Client",
    "AsyncClient",
]


class Fireworks(SyncAPIClient):
    # client options
    api_key: str
    account_id: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Fireworks client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `FIREWORKS_API_KEY`
        - `account_id` from `FIREWORKS_ACCOUNT_ID`
        """
        if api_key is None:
            api_key = os.environ.get("FIREWORKS_API_KEY")
        if api_key is None:
            raise FireworksError(
                "The api_key client option must be set either by passing api_key to the client or by setting the FIREWORKS_API_KEY environment variable"
            )
        self.api_key = api_key

        if account_id is None:
            account_id = os.environ.get("FIREWORKS_ACCOUNT_ID")
        self.account_id = account_id

        if base_url is None:
            base_url = os.environ.get("FIREWORKS_BASE_URL")
        self._base_url_overridden = base_url is not None
        if base_url is None:
            base_url = f"https://api.fireworks.ai"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._default_stream_cls = Stream

    @cached_property
    def chat(self) -> ChatResource:
        from .resources.chat import ChatResource

        return ChatResource(self)

    @cached_property
    def completions(self) -> CompletionsResource:
        from .resources.completions import CompletionsResource

        return CompletionsResource(self)

    @cached_property
    def messages(self) -> MessagesResource:
        from .resources.messages import MessagesResource

        return MessagesResource(self)

    @cached_property
    def batch_inference_jobs(self) -> BatchInferenceJobsResource:
        from .resources.batch_inference_jobs import BatchInferenceJobsResource

        return BatchInferenceJobsResource(self)

    @cached_property
    def deployments(self) -> DeploymentsResource:
        from .resources.deployments import DeploymentsResource

        return DeploymentsResource(self)

    @cached_property
    def models(self) -> ModelsResource:
        from .resources.models import ModelsResource

        return ModelsResource(self)

    @cached_property
    def lora(self) -> LoraResource:
        from .resources.lora import LoraResource

        return LoraResource(self)

    @cached_property
    def deployment_shapes(self) -> DeploymentShapesResource:
        from .resources.deployment_shapes import DeploymentShapesResource

        return DeploymentShapesResource(self)

    @cached_property
    def deployment_shape_versions(self) -> DeploymentShapeVersionsResource:
        from .resources.deployment_shape_versions import DeploymentShapeVersionsResource

        return DeploymentShapeVersionsResource(self)

    @cached_property
    def datasets(self) -> DatasetsResource:
        from .resources.datasets import DatasetsResource

        return DatasetsResource(self)

    @cached_property
    def supervised_fine_tuning_jobs(self) -> SupervisedFineTuningJobsResource:
        from .resources.supervised_fine_tuning_jobs import SupervisedFineTuningJobsResource

        return SupervisedFineTuningJobsResource(self)

    @cached_property
    def reinforcement_fine_tuning_jobs(self) -> ReinforcementFineTuningJobsResource:
        from .resources.reinforcement_fine_tuning_jobs import ReinforcementFineTuningJobsResource

        return ReinforcementFineTuningJobsResource(self)

    @cached_property
    def reinforcement_fine_tuning_steps(self) -> ReinforcementFineTuningStepsResource:
        from .resources.reinforcement_fine_tuning_steps import ReinforcementFineTuningStepsResource

        return ReinforcementFineTuningStepsResource(self)

    @cached_property
    def dpo_jobs(self) -> DpoJobsResource:
        from .resources.dpo_jobs import DpoJobsResource

        return DpoJobsResource(self)

    @cached_property
    def evaluation_jobs(self) -> EvaluationJobsResource:
        from .resources.evaluation_jobs import EvaluationJobsResource

        return EvaluationJobsResource(self)

    @cached_property
    def evaluators(self) -> EvaluatorsResource:
        from .resources.evaluators import EvaluatorsResource

        return EvaluatorsResource(self)

    @cached_property
    def accounts(self) -> AccountsResource:
        from .resources.accounts import AccountsResource

        return AccountsResource(self)

    @cached_property
    def users(self) -> UsersResource:
        from .resources.users import UsersResource

        return UsersResource(self)

    @cached_property
    def api_keys(self) -> APIKeysResource:
        from .resources.api_keys import APIKeysResource

        return APIKeysResource(self)

    @cached_property
    def secrets(self) -> SecretsResource:
        from .resources.secrets import SecretsResource

        return SecretsResource(self)

    @cached_property
    def with_raw_response(self) -> FireworksWithRawResponse:
        return FireworksWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FireworksWithStreamedResponse:
        return FireworksWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        client = self.__class__(
            api_key=api_key or self.api_key,
            account_id=account_id or self.account_id,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )
        client._base_url_overridden = self._base_url_overridden or base_url is not None
        return client

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    def _get_account_id_path_param(self) -> str:
        from_client = self.account_id
        if from_client is not None:
            return from_client

        raise ValueError(
            "Missing account_id argument; Please provide it at the client level, e.g. Fireworks(account_id='abcd') or per method."
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncFireworks(AsyncAPIClient):
    # client options
    api_key: str
    account_id: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncFireworks client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `FIREWORKS_API_KEY`
        - `account_id` from `FIREWORKS_ACCOUNT_ID`
        """
        if api_key is None:
            api_key = os.environ.get("FIREWORKS_API_KEY")
        if api_key is None:
            raise FireworksError(
                "The api_key client option must be set either by passing api_key to the client or by setting the FIREWORKS_API_KEY environment variable"
            )
        self.api_key = api_key

        if account_id is None:
            account_id = os.environ.get("FIREWORKS_ACCOUNT_ID")
        self.account_id = account_id

        if base_url is None:
            base_url = os.environ.get("FIREWORKS_BASE_URL")
        self._base_url_overridden = base_url is not None
        if base_url is None:
            base_url = f"https://api.fireworks.ai"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._default_stream_cls = AsyncStream

    @cached_property
    def chat(self) -> AsyncChatResource:
        from .resources.chat import AsyncChatResource

        return AsyncChatResource(self)

    @cached_property
    def completions(self) -> AsyncCompletionsResource:
        from .resources.completions import AsyncCompletionsResource

        return AsyncCompletionsResource(self)

    @cached_property
    def messages(self) -> AsyncMessagesResource:
        from .resources.messages import AsyncMessagesResource

        return AsyncMessagesResource(self)

    @cached_property
    def batch_inference_jobs(self) -> AsyncBatchInferenceJobsResource:
        from .resources.batch_inference_jobs import AsyncBatchInferenceJobsResource

        return AsyncBatchInferenceJobsResource(self)

    @cached_property
    def deployments(self) -> AsyncDeploymentsResource:
        from .resources.deployments import AsyncDeploymentsResource

        return AsyncDeploymentsResource(self)

    @cached_property
    def models(self) -> AsyncModelsResource:
        from .resources.models import AsyncModelsResource

        return AsyncModelsResource(self)

    @cached_property
    def lora(self) -> AsyncLoraResource:
        from .resources.lora import AsyncLoraResource

        return AsyncLoraResource(self)

    @cached_property
    def deployment_shapes(self) -> AsyncDeploymentShapesResource:
        from .resources.deployment_shapes import AsyncDeploymentShapesResource

        return AsyncDeploymentShapesResource(self)

    @cached_property
    def deployment_shape_versions(self) -> AsyncDeploymentShapeVersionsResource:
        from .resources.deployment_shape_versions import AsyncDeploymentShapeVersionsResource

        return AsyncDeploymentShapeVersionsResource(self)

    @cached_property
    def datasets(self) -> AsyncDatasetsResource:
        from .resources.datasets import AsyncDatasetsResource

        return AsyncDatasetsResource(self)

    @cached_property
    def supervised_fine_tuning_jobs(self) -> AsyncSupervisedFineTuningJobsResource:
        from .resources.supervised_fine_tuning_jobs import AsyncSupervisedFineTuningJobsResource

        return AsyncSupervisedFineTuningJobsResource(self)

    @cached_property
    def reinforcement_fine_tuning_jobs(self) -> AsyncReinforcementFineTuningJobsResource:
        from .resources.reinforcement_fine_tuning_jobs import AsyncReinforcementFineTuningJobsResource

        return AsyncReinforcementFineTuningJobsResource(self)

    @cached_property
    def reinforcement_fine_tuning_steps(self) -> AsyncReinforcementFineTuningStepsResource:
        from .resources.reinforcement_fine_tuning_steps import AsyncReinforcementFineTuningStepsResource

        return AsyncReinforcementFineTuningStepsResource(self)

    @cached_property
    def dpo_jobs(self) -> AsyncDpoJobsResource:
        from .resources.dpo_jobs import AsyncDpoJobsResource

        return AsyncDpoJobsResource(self)

    @cached_property
    def evaluation_jobs(self) -> AsyncEvaluationJobsResource:
        from .resources.evaluation_jobs import AsyncEvaluationJobsResource

        return AsyncEvaluationJobsResource(self)

    @cached_property
    def evaluators(self) -> AsyncEvaluatorsResource:
        from .resources.evaluators import AsyncEvaluatorsResource

        return AsyncEvaluatorsResource(self)

    @cached_property
    def accounts(self) -> AsyncAccountsResource:
        from .resources.accounts import AsyncAccountsResource

        return AsyncAccountsResource(self)

    @cached_property
    def users(self) -> AsyncUsersResource:
        from .resources.users import AsyncUsersResource

        return AsyncUsersResource(self)

    @cached_property
    def api_keys(self) -> AsyncAPIKeysResource:
        from .resources.api_keys import AsyncAPIKeysResource

        return AsyncAPIKeysResource(self)

    @cached_property
    def secrets(self) -> AsyncSecretsResource:
        from .resources.secrets import AsyncSecretsResource

        return AsyncSecretsResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncFireworksWithRawResponse:
        return AsyncFireworksWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFireworksWithStreamedResponse:
        return AsyncFireworksWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        client = self.__class__(
            api_key=api_key or self.api_key,
            account_id=account_id or self.account_id,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )
        client._base_url_overridden = self._base_url_overridden or base_url is not None
        return client

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    def _get_account_id_path_param(self) -> str:
        from_client = self.account_id
        if from_client is not None:
            return from_client

        raise ValueError(
            "Missing account_id argument; Please provide it at the client level, e.g. AsyncFireworks(account_id='abcd') or per method."
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class FireworksWithRawResponse:
    _client: Fireworks

    def __init__(self, client: Fireworks) -> None:
        self._client = client

    @cached_property
    def chat(self) -> chat.ChatResourceWithRawResponse:
        from .resources.chat import ChatResourceWithRawResponse

        return ChatResourceWithRawResponse(self._client.chat)

    @cached_property
    def completions(self) -> completions.CompletionsResourceWithRawResponse:
        from .resources.completions import CompletionsResourceWithRawResponse

        return CompletionsResourceWithRawResponse(self._client.completions)

    @cached_property
    def messages(self) -> messages.MessagesResourceWithRawResponse:
        from .resources.messages import MessagesResourceWithRawResponse

        return MessagesResourceWithRawResponse(self._client.messages)

    @cached_property
    def batch_inference_jobs(self) -> batch_inference_jobs.BatchInferenceJobsResourceWithRawResponse:
        from .resources.batch_inference_jobs import BatchInferenceJobsResourceWithRawResponse

        return BatchInferenceJobsResourceWithRawResponse(self._client.batch_inference_jobs)

    @cached_property
    def deployments(self) -> deployments.DeploymentsResourceWithRawResponse:
        from .resources.deployments import DeploymentsResourceWithRawResponse

        return DeploymentsResourceWithRawResponse(self._client.deployments)

    @cached_property
    def models(self) -> models.ModelsResourceWithRawResponse:
        from .resources.models import ModelsResourceWithRawResponse

        return ModelsResourceWithRawResponse(self._client.models)

    @cached_property
    def lora(self) -> lora.LoraResourceWithRawResponse:
        from .resources.lora import LoraResourceWithRawResponse

        return LoraResourceWithRawResponse(self._client.lora)

    @cached_property
    def deployment_shapes(self) -> deployment_shapes.DeploymentShapesResourceWithRawResponse:
        from .resources.deployment_shapes import DeploymentShapesResourceWithRawResponse

        return DeploymentShapesResourceWithRawResponse(self._client.deployment_shapes)

    @cached_property
    def deployment_shape_versions(self) -> deployment_shape_versions.DeploymentShapeVersionsResourceWithRawResponse:
        from .resources.deployment_shape_versions import DeploymentShapeVersionsResourceWithRawResponse

        return DeploymentShapeVersionsResourceWithRawResponse(self._client.deployment_shape_versions)

    @cached_property
    def datasets(self) -> datasets.DatasetsResourceWithRawResponse:
        from .resources.datasets import DatasetsResourceWithRawResponse

        return DatasetsResourceWithRawResponse(self._client.datasets)

    @cached_property
    def supervised_fine_tuning_jobs(
        self,
    ) -> supervised_fine_tuning_jobs.SupervisedFineTuningJobsResourceWithRawResponse:
        from .resources.supervised_fine_tuning_jobs import SupervisedFineTuningJobsResourceWithRawResponse

        return SupervisedFineTuningJobsResourceWithRawResponse(self._client.supervised_fine_tuning_jobs)

    @cached_property
    def reinforcement_fine_tuning_jobs(
        self,
    ) -> reinforcement_fine_tuning_jobs.ReinforcementFineTuningJobsResourceWithRawResponse:
        from .resources.reinforcement_fine_tuning_jobs import ReinforcementFineTuningJobsResourceWithRawResponse

        return ReinforcementFineTuningJobsResourceWithRawResponse(self._client.reinforcement_fine_tuning_jobs)

    @cached_property
    def reinforcement_fine_tuning_steps(
        self,
    ) -> reinforcement_fine_tuning_steps.ReinforcementFineTuningStepsResourceWithRawResponse:
        from .resources.reinforcement_fine_tuning_steps import ReinforcementFineTuningStepsResourceWithRawResponse

        return ReinforcementFineTuningStepsResourceWithRawResponse(self._client.reinforcement_fine_tuning_steps)

    @cached_property
    def dpo_jobs(self) -> dpo_jobs.DpoJobsResourceWithRawResponse:
        from .resources.dpo_jobs import DpoJobsResourceWithRawResponse

        return DpoJobsResourceWithRawResponse(self._client.dpo_jobs)

    @cached_property
    def evaluation_jobs(self) -> evaluation_jobs.EvaluationJobsResourceWithRawResponse:
        from .resources.evaluation_jobs import EvaluationJobsResourceWithRawResponse

        return EvaluationJobsResourceWithRawResponse(self._client.evaluation_jobs)

    @cached_property
    def evaluators(self) -> evaluators.EvaluatorsResourceWithRawResponse:
        from .resources.evaluators import EvaluatorsResourceWithRawResponse

        return EvaluatorsResourceWithRawResponse(self._client.evaluators)

    @cached_property
    def accounts(self) -> accounts.AccountsResourceWithRawResponse:
        from .resources.accounts import AccountsResourceWithRawResponse

        return AccountsResourceWithRawResponse(self._client.accounts)

    @cached_property
    def users(self) -> users.UsersResourceWithRawResponse:
        from .resources.users import UsersResourceWithRawResponse

        return UsersResourceWithRawResponse(self._client.users)

    @cached_property
    def api_keys(self) -> api_keys.APIKeysResourceWithRawResponse:
        from .resources.api_keys import APIKeysResourceWithRawResponse

        return APIKeysResourceWithRawResponse(self._client.api_keys)

    @cached_property
    def secrets(self) -> secrets.SecretsResourceWithRawResponse:
        from .resources.secrets import SecretsResourceWithRawResponse

        return SecretsResourceWithRawResponse(self._client.secrets)


class AsyncFireworksWithRawResponse:
    _client: AsyncFireworks

    def __init__(self, client: AsyncFireworks) -> None:
        self._client = client

    @cached_property
    def chat(self) -> chat.AsyncChatResourceWithRawResponse:
        from .resources.chat import AsyncChatResourceWithRawResponse

        return AsyncChatResourceWithRawResponse(self._client.chat)

    @cached_property
    def completions(self) -> completions.AsyncCompletionsResourceWithRawResponse:
        from .resources.completions import AsyncCompletionsResourceWithRawResponse

        return AsyncCompletionsResourceWithRawResponse(self._client.completions)

    @cached_property
    def messages(self) -> messages.AsyncMessagesResourceWithRawResponse:
        from .resources.messages import AsyncMessagesResourceWithRawResponse

        return AsyncMessagesResourceWithRawResponse(self._client.messages)

    @cached_property
    def batch_inference_jobs(self) -> batch_inference_jobs.AsyncBatchInferenceJobsResourceWithRawResponse:
        from .resources.batch_inference_jobs import AsyncBatchInferenceJobsResourceWithRawResponse

        return AsyncBatchInferenceJobsResourceWithRawResponse(self._client.batch_inference_jobs)

    @cached_property
    def deployments(self) -> deployments.AsyncDeploymentsResourceWithRawResponse:
        from .resources.deployments import AsyncDeploymentsResourceWithRawResponse

        return AsyncDeploymentsResourceWithRawResponse(self._client.deployments)

    @cached_property
    def models(self) -> models.AsyncModelsResourceWithRawResponse:
        from .resources.models import AsyncModelsResourceWithRawResponse

        return AsyncModelsResourceWithRawResponse(self._client.models)

    @cached_property
    def lora(self) -> lora.AsyncLoraResourceWithRawResponse:
        from .resources.lora import AsyncLoraResourceWithRawResponse

        return AsyncLoraResourceWithRawResponse(self._client.lora)

    @cached_property
    def deployment_shapes(self) -> deployment_shapes.AsyncDeploymentShapesResourceWithRawResponse:
        from .resources.deployment_shapes import AsyncDeploymentShapesResourceWithRawResponse

        return AsyncDeploymentShapesResourceWithRawResponse(self._client.deployment_shapes)

    @cached_property
    def deployment_shape_versions(
        self,
    ) -> deployment_shape_versions.AsyncDeploymentShapeVersionsResourceWithRawResponse:
        from .resources.deployment_shape_versions import AsyncDeploymentShapeVersionsResourceWithRawResponse

        return AsyncDeploymentShapeVersionsResourceWithRawResponse(self._client.deployment_shape_versions)

    @cached_property
    def datasets(self) -> datasets.AsyncDatasetsResourceWithRawResponse:
        from .resources.datasets import AsyncDatasetsResourceWithRawResponse

        return AsyncDatasetsResourceWithRawResponse(self._client.datasets)

    @cached_property
    def supervised_fine_tuning_jobs(
        self,
    ) -> supervised_fine_tuning_jobs.AsyncSupervisedFineTuningJobsResourceWithRawResponse:
        from .resources.supervised_fine_tuning_jobs import AsyncSupervisedFineTuningJobsResourceWithRawResponse

        return AsyncSupervisedFineTuningJobsResourceWithRawResponse(self._client.supervised_fine_tuning_jobs)

    @cached_property
    def reinforcement_fine_tuning_jobs(
        self,
    ) -> reinforcement_fine_tuning_jobs.AsyncReinforcementFineTuningJobsResourceWithRawResponse:
        from .resources.reinforcement_fine_tuning_jobs import AsyncReinforcementFineTuningJobsResourceWithRawResponse

        return AsyncReinforcementFineTuningJobsResourceWithRawResponse(self._client.reinforcement_fine_tuning_jobs)

    @cached_property
    def reinforcement_fine_tuning_steps(
        self,
    ) -> reinforcement_fine_tuning_steps.AsyncReinforcementFineTuningStepsResourceWithRawResponse:
        from .resources.reinforcement_fine_tuning_steps import AsyncReinforcementFineTuningStepsResourceWithRawResponse

        return AsyncReinforcementFineTuningStepsResourceWithRawResponse(self._client.reinforcement_fine_tuning_steps)

    @cached_property
    def dpo_jobs(self) -> dpo_jobs.AsyncDpoJobsResourceWithRawResponse:
        from .resources.dpo_jobs import AsyncDpoJobsResourceWithRawResponse

        return AsyncDpoJobsResourceWithRawResponse(self._client.dpo_jobs)

    @cached_property
    def evaluation_jobs(self) -> evaluation_jobs.AsyncEvaluationJobsResourceWithRawResponse:
        from .resources.evaluation_jobs import AsyncEvaluationJobsResourceWithRawResponse

        return AsyncEvaluationJobsResourceWithRawResponse(self._client.evaluation_jobs)

    @cached_property
    def evaluators(self) -> evaluators.AsyncEvaluatorsResourceWithRawResponse:
        from .resources.evaluators import AsyncEvaluatorsResourceWithRawResponse

        return AsyncEvaluatorsResourceWithRawResponse(self._client.evaluators)

    @cached_property
    def accounts(self) -> accounts.AsyncAccountsResourceWithRawResponse:
        from .resources.accounts import AsyncAccountsResourceWithRawResponse

        return AsyncAccountsResourceWithRawResponse(self._client.accounts)

    @cached_property
    def users(self) -> users.AsyncUsersResourceWithRawResponse:
        from .resources.users import AsyncUsersResourceWithRawResponse

        return AsyncUsersResourceWithRawResponse(self._client.users)

    @cached_property
    def api_keys(self) -> api_keys.AsyncAPIKeysResourceWithRawResponse:
        from .resources.api_keys import AsyncAPIKeysResourceWithRawResponse

        return AsyncAPIKeysResourceWithRawResponse(self._client.api_keys)

    @cached_property
    def secrets(self) -> secrets.AsyncSecretsResourceWithRawResponse:
        from .resources.secrets import AsyncSecretsResourceWithRawResponse

        return AsyncSecretsResourceWithRawResponse(self._client.secrets)


class FireworksWithStreamedResponse:
    _client: Fireworks

    def __init__(self, client: Fireworks) -> None:
        self._client = client

    @cached_property
    def chat(self) -> chat.ChatResourceWithStreamingResponse:
        from .resources.chat import ChatResourceWithStreamingResponse

        return ChatResourceWithStreamingResponse(self._client.chat)

    @cached_property
    def completions(self) -> completions.CompletionsResourceWithStreamingResponse:
        from .resources.completions import CompletionsResourceWithStreamingResponse

        return CompletionsResourceWithStreamingResponse(self._client.completions)

    @cached_property
    def messages(self) -> messages.MessagesResourceWithStreamingResponse:
        from .resources.messages import MessagesResourceWithStreamingResponse

        return MessagesResourceWithStreamingResponse(self._client.messages)

    @cached_property
    def batch_inference_jobs(self) -> batch_inference_jobs.BatchInferenceJobsResourceWithStreamingResponse:
        from .resources.batch_inference_jobs import BatchInferenceJobsResourceWithStreamingResponse

        return BatchInferenceJobsResourceWithStreamingResponse(self._client.batch_inference_jobs)

    @cached_property
    def deployments(self) -> deployments.DeploymentsResourceWithStreamingResponse:
        from .resources.deployments import DeploymentsResourceWithStreamingResponse

        return DeploymentsResourceWithStreamingResponse(self._client.deployments)

    @cached_property
    def models(self) -> models.ModelsResourceWithStreamingResponse:
        from .resources.models import ModelsResourceWithStreamingResponse

        return ModelsResourceWithStreamingResponse(self._client.models)

    @cached_property
    def lora(self) -> lora.LoraResourceWithStreamingResponse:
        from .resources.lora import LoraResourceWithStreamingResponse

        return LoraResourceWithStreamingResponse(self._client.lora)

    @cached_property
    def deployment_shapes(self) -> deployment_shapes.DeploymentShapesResourceWithStreamingResponse:
        from .resources.deployment_shapes import DeploymentShapesResourceWithStreamingResponse

        return DeploymentShapesResourceWithStreamingResponse(self._client.deployment_shapes)

    @cached_property
    def deployment_shape_versions(
        self,
    ) -> deployment_shape_versions.DeploymentShapeVersionsResourceWithStreamingResponse:
        from .resources.deployment_shape_versions import DeploymentShapeVersionsResourceWithStreamingResponse

        return DeploymentShapeVersionsResourceWithStreamingResponse(self._client.deployment_shape_versions)

    @cached_property
    def datasets(self) -> datasets.DatasetsResourceWithStreamingResponse:
        from .resources.datasets import DatasetsResourceWithStreamingResponse

        return DatasetsResourceWithStreamingResponse(self._client.datasets)

    @cached_property
    def supervised_fine_tuning_jobs(
        self,
    ) -> supervised_fine_tuning_jobs.SupervisedFineTuningJobsResourceWithStreamingResponse:
        from .resources.supervised_fine_tuning_jobs import SupervisedFineTuningJobsResourceWithStreamingResponse

        return SupervisedFineTuningJobsResourceWithStreamingResponse(self._client.supervised_fine_tuning_jobs)

    @cached_property
    def reinforcement_fine_tuning_jobs(
        self,
    ) -> reinforcement_fine_tuning_jobs.ReinforcementFineTuningJobsResourceWithStreamingResponse:
        from .resources.reinforcement_fine_tuning_jobs import ReinforcementFineTuningJobsResourceWithStreamingResponse

        return ReinforcementFineTuningJobsResourceWithStreamingResponse(self._client.reinforcement_fine_tuning_jobs)

    @cached_property
    def reinforcement_fine_tuning_steps(
        self,
    ) -> reinforcement_fine_tuning_steps.ReinforcementFineTuningStepsResourceWithStreamingResponse:
        from .resources.reinforcement_fine_tuning_steps import ReinforcementFineTuningStepsResourceWithStreamingResponse

        return ReinforcementFineTuningStepsResourceWithStreamingResponse(self._client.reinforcement_fine_tuning_steps)

    @cached_property
    def dpo_jobs(self) -> dpo_jobs.DpoJobsResourceWithStreamingResponse:
        from .resources.dpo_jobs import DpoJobsResourceWithStreamingResponse

        return DpoJobsResourceWithStreamingResponse(self._client.dpo_jobs)

    @cached_property
    def evaluation_jobs(self) -> evaluation_jobs.EvaluationJobsResourceWithStreamingResponse:
        from .resources.evaluation_jobs import EvaluationJobsResourceWithStreamingResponse

        return EvaluationJobsResourceWithStreamingResponse(self._client.evaluation_jobs)

    @cached_property
    def evaluators(self) -> evaluators.EvaluatorsResourceWithStreamingResponse:
        from .resources.evaluators import EvaluatorsResourceWithStreamingResponse

        return EvaluatorsResourceWithStreamingResponse(self._client.evaluators)

    @cached_property
    def accounts(self) -> accounts.AccountsResourceWithStreamingResponse:
        from .resources.accounts import AccountsResourceWithStreamingResponse

        return AccountsResourceWithStreamingResponse(self._client.accounts)

    @cached_property
    def users(self) -> users.UsersResourceWithStreamingResponse:
        from .resources.users import UsersResourceWithStreamingResponse

        return UsersResourceWithStreamingResponse(self._client.users)

    @cached_property
    def api_keys(self) -> api_keys.APIKeysResourceWithStreamingResponse:
        from .resources.api_keys import APIKeysResourceWithStreamingResponse

        return APIKeysResourceWithStreamingResponse(self._client.api_keys)

    @cached_property
    def secrets(self) -> secrets.SecretsResourceWithStreamingResponse:
        from .resources.secrets import SecretsResourceWithStreamingResponse

        return SecretsResourceWithStreamingResponse(self._client.secrets)


class AsyncFireworksWithStreamedResponse:
    _client: AsyncFireworks

    def __init__(self, client: AsyncFireworks) -> None:
        self._client = client

    @cached_property
    def chat(self) -> chat.AsyncChatResourceWithStreamingResponse:
        from .resources.chat import AsyncChatResourceWithStreamingResponse

        return AsyncChatResourceWithStreamingResponse(self._client.chat)

    @cached_property
    def completions(self) -> completions.AsyncCompletionsResourceWithStreamingResponse:
        from .resources.completions import AsyncCompletionsResourceWithStreamingResponse

        return AsyncCompletionsResourceWithStreamingResponse(self._client.completions)

    @cached_property
    def messages(self) -> messages.AsyncMessagesResourceWithStreamingResponse:
        from .resources.messages import AsyncMessagesResourceWithStreamingResponse

        return AsyncMessagesResourceWithStreamingResponse(self._client.messages)

    @cached_property
    def batch_inference_jobs(self) -> batch_inference_jobs.AsyncBatchInferenceJobsResourceWithStreamingResponse:
        from .resources.batch_inference_jobs import AsyncBatchInferenceJobsResourceWithStreamingResponse

        return AsyncBatchInferenceJobsResourceWithStreamingResponse(self._client.batch_inference_jobs)

    @cached_property
    def deployments(self) -> deployments.AsyncDeploymentsResourceWithStreamingResponse:
        from .resources.deployments import AsyncDeploymentsResourceWithStreamingResponse

        return AsyncDeploymentsResourceWithStreamingResponse(self._client.deployments)

    @cached_property
    def models(self) -> models.AsyncModelsResourceWithStreamingResponse:
        from .resources.models import AsyncModelsResourceWithStreamingResponse

        return AsyncModelsResourceWithStreamingResponse(self._client.models)

    @cached_property
    def lora(self) -> lora.AsyncLoraResourceWithStreamingResponse:
        from .resources.lora import AsyncLoraResourceWithStreamingResponse

        return AsyncLoraResourceWithStreamingResponse(self._client.lora)

    @cached_property
    def deployment_shapes(self) -> deployment_shapes.AsyncDeploymentShapesResourceWithStreamingResponse:
        from .resources.deployment_shapes import AsyncDeploymentShapesResourceWithStreamingResponse

        return AsyncDeploymentShapesResourceWithStreamingResponse(self._client.deployment_shapes)

    @cached_property
    def deployment_shape_versions(
        self,
    ) -> deployment_shape_versions.AsyncDeploymentShapeVersionsResourceWithStreamingResponse:
        from .resources.deployment_shape_versions import AsyncDeploymentShapeVersionsResourceWithStreamingResponse

        return AsyncDeploymentShapeVersionsResourceWithStreamingResponse(self._client.deployment_shape_versions)

    @cached_property
    def datasets(self) -> datasets.AsyncDatasetsResourceWithStreamingResponse:
        from .resources.datasets import AsyncDatasetsResourceWithStreamingResponse

        return AsyncDatasetsResourceWithStreamingResponse(self._client.datasets)

    @cached_property
    def supervised_fine_tuning_jobs(
        self,
    ) -> supervised_fine_tuning_jobs.AsyncSupervisedFineTuningJobsResourceWithStreamingResponse:
        from .resources.supervised_fine_tuning_jobs import AsyncSupervisedFineTuningJobsResourceWithStreamingResponse

        return AsyncSupervisedFineTuningJobsResourceWithStreamingResponse(self._client.supervised_fine_tuning_jobs)

    @cached_property
    def reinforcement_fine_tuning_jobs(
        self,
    ) -> reinforcement_fine_tuning_jobs.AsyncReinforcementFineTuningJobsResourceWithStreamingResponse:
        from .resources.reinforcement_fine_tuning_jobs import (
            AsyncReinforcementFineTuningJobsResourceWithStreamingResponse,
        )

        return AsyncReinforcementFineTuningJobsResourceWithStreamingResponse(
            self._client.reinforcement_fine_tuning_jobs
        )

    @cached_property
    def reinforcement_fine_tuning_steps(
        self,
    ) -> reinforcement_fine_tuning_steps.AsyncReinforcementFineTuningStepsResourceWithStreamingResponse:
        from .resources.reinforcement_fine_tuning_steps import (
            AsyncReinforcementFineTuningStepsResourceWithStreamingResponse,
        )

        return AsyncReinforcementFineTuningStepsResourceWithStreamingResponse(
            self._client.reinforcement_fine_tuning_steps
        )

    @cached_property
    def dpo_jobs(self) -> dpo_jobs.AsyncDpoJobsResourceWithStreamingResponse:
        from .resources.dpo_jobs import AsyncDpoJobsResourceWithStreamingResponse

        return AsyncDpoJobsResourceWithStreamingResponse(self._client.dpo_jobs)

    @cached_property
    def evaluation_jobs(self) -> evaluation_jobs.AsyncEvaluationJobsResourceWithStreamingResponse:
        from .resources.evaluation_jobs import AsyncEvaluationJobsResourceWithStreamingResponse

        return AsyncEvaluationJobsResourceWithStreamingResponse(self._client.evaluation_jobs)

    @cached_property
    def evaluators(self) -> evaluators.AsyncEvaluatorsResourceWithStreamingResponse:
        from .resources.evaluators import AsyncEvaluatorsResourceWithStreamingResponse

        return AsyncEvaluatorsResourceWithStreamingResponse(self._client.evaluators)

    @cached_property
    def accounts(self) -> accounts.AsyncAccountsResourceWithStreamingResponse:
        from .resources.accounts import AsyncAccountsResourceWithStreamingResponse

        return AsyncAccountsResourceWithStreamingResponse(self._client.accounts)

    @cached_property
    def users(self) -> users.AsyncUsersResourceWithStreamingResponse:
        from .resources.users import AsyncUsersResourceWithStreamingResponse

        return AsyncUsersResourceWithStreamingResponse(self._client.users)

    @cached_property
    def api_keys(self) -> api_keys.AsyncAPIKeysResourceWithStreamingResponse:
        from .resources.api_keys import AsyncAPIKeysResourceWithStreamingResponse

        return AsyncAPIKeysResourceWithStreamingResponse(self._client.api_keys)

    @cached_property
    def secrets(self) -> secrets.AsyncSecretsResourceWithStreamingResponse:
        from .resources.secrets import AsyncSecretsResourceWithStreamingResponse

        return AsyncSecretsResourceWithStreamingResponse(self._client.secrets)


Client = Fireworks

AsyncClient = AsyncFireworks
